using System;
using System.Collections.Generic;
using System.Net.Mime;
using System.Security.Cryptography;
using FarseerPhysics.Dynamics.Joints;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using Silk.NET.SDL;
using Color = Jypeli.Color;

namespace KelaCombatTesti;

/// @author Sakari
/// @version 17.02.2025
/// <summary>
/// 
/// </summary>
    public class KelaCombatTesti : PhysicsGame
    {
        int kenttanumero = 0;
        string[] Kentat = {"kentta1", "kentta2"};
        Image[] HenkiloSaapastelu = LoadImages("kavely1", "kavely2");
        Image[] PaikallaanAnimaatio = LoadImages("kavely1");
        Image[] PutoamisAnimaatio = LoadImages("putoaminen1", "putoaminen2", "putoaminen3", "putoaminen4", "putoaminen5");
        Image[] KelabottiKavely = LoadImages("Kelabotti", "Kelabotti2");
        
        SoundEffect Hyppyaani = LoadSoundEffect("Hyppyaani");
        SoundEffect Viinapulloaani = LoadSoundEffect("Viinapulloaani");
        SoundEffect Keladamageaani = LoadSoundEffect("Keladamageaani");
        SoundEffect Kavelyaani = LoadSoundEffect("Kavelyaani");
        SoundEffect Lyontiaani = LoadSoundEffect("Lyontiaani");
        Timer Ajastin = new Timer();

        PlatformCharacter pelaaja;
        PhysicsObject Pesismaila;
        PhysicsObject Viinapullo;
        PhysicsObject kappale;
        PhysicsObject kolmio;
        PlatformCharacter Kelabotti;
        Image TiiliseinaKuva = LoadImage("tiiliseina");
        Image KelabottiKuva = LoadImage("Kelabotti");
        //Image PelaajaKuva = LoadImage("Pelaaja.png");
        Image ViinapulloKuva = LoadImage("Viinapullo.png");
        
        List<PhysicsObject> spawnipaikat = new List<PhysicsObject>(); // tässä listataan mahdolliset paikat, johon kelabotti voi spawnaa. Lista pitää sisällään spawnereita.
        
        //IntMeter edustaa muuttujia, jotka voivat kasvaa tai vähentyä tietyllä tavalla 
        IntMeter pisteet;  //pelaajan keräämät pisteet 
        DoubleMeter pelaajanelamaLaskuri; //pelaajan elämät
        IntMeter Kelabotinelamalaskuri; //kelabotin elämät
        
        //Label = tekstielementti, joka näytetään pelissä 
        Label pisteNaytto; //pisteiden näyttäminen ruudulla 

        
        
      
        /// <summary>
        /// Pedli alkaa tästä
        /// </summary>
        public override void Begin()
        {
            LuoKentta(0);
        }
        
        /// <summary>
        /// Vasemmalle kävely
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void LiikuVasemmalle(Vector vektori)
        {
            pelaaja.Walk(-120);
            
        }

        /// <summary>
        /// Vasemmalle kävedllessä kuuluva ääni
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void AaniVasemmalle(Vector vektori)
        {
            Kavelyaani.Play();
            Ajastin.Interval = 0.5; // Äänen kesto
            Ajastin.Timeout += delegate { Kavelyaani.Play(); };
            Ajastin.Start();
        }

        /// <summary>
        /// Vasemmalle suuntautuvan äänen lopettaminen
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void LopetaAaniVasemmalle(Vector vektori)
        {
            if (Ajastin != null) // Tarkistetaan, onko ajastin luotu
            {
                Ajastin.Stop(); // Pysäytetään ajastin
                Kavelyaani.Stop(); // Lopetetaan äänen toisto
            }
        }
        
        /// <summary>
        /// Oikealle kävely
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void LiikuOikealle(Vector vektori)
        {
            pelaaja.Walk(120);
            
        }
        
        /// <summary>
        /// Ääni oikealle liikkuessa
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void AaniOikealle(Vector vektori)
        {
            Kavelyaani.Play();
            Ajastin.Interval = 0.5; // Äänen kesto
            Ajastin.Timeout += delegate { Kavelyaani.Play(); };
            Ajastin.Start();
        }

        /// <summary>
        /// Oikealle suuntautuvan äänen lopettaminen
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void LopetaAaniOikealle(Vector vektori)
        {
            if (Ajastin != null) // Tarkistetaan, onko ajastin luotu
            {
                Ajastin.Stop(); // Pysäytetään ajastin
                Kavelyaani.Stop(); // Lopetetaan äänen toisto
            }
        }
        
        /// <summary>
        /// Hyppääminen
        /// </summary>
        /// <param name="vektori">suuntavektori</param>
        void Hyppaa(Vector vektori)
        {
            Hyppyaani.Play();
            pelaaja.Jump(900.0);
        }
        
        /// <summary>
        /// Tässä luodaan pelikenttä
        /// </summary>
        void LuoKentta(int numero)
        {
            if (kenttanumero == Kentat.Length)
            {
                Label Valmistuminen = new Label(800, 60, "Toimeentulohakemus hyväkystty.");
                Valmistuminen.Y = 140;
                Valmistuminen.Font = new Font(60);
                Valmistuminen.TextColor = Color.LimeGreen;
                Add(Valmistuminen);
            }
        else
        {
            ClearAll();
            
            Level.Background.CreateGradient(Color.DarkGray, Color.Blue);
            Gravity = new Vector(0, -1100.0);
           
            // pelissä kentät luodaan piirtämällä pikseleitä. Pikselin väri kuvaa aina jotain tiettyä asiaa kentällä
            // ruutu = pikseli
            ColorTileMap ruudut = ColorTileMap.FromLevelAsset(Kentat[numero]);
            
            ruudut.SetTileMethod(Color.Lime, LuoPelaaja); //LIME pikseli on pelaaja,  0, 255, 0
            ruudut.SetTileMethod(Color.Black, LuoTaso); //MUSTA on tiiliseinää
            ruudut.SetTileMethod(Color.Red, LuoKelabottiSpawni); //PUNAISEEN voi spawnaa vihu
            ruudut.SetTileMethod(Color.SkyBlue, LuoViinapullo); //VAALEANSNISET on pelaajaa parantavia viinapulloja 135, 206, 235
            ruudut.SetTileMethod(Color.Wheat, LuoExit); //  245, 222, 179
            
            ruudut.Execute(80, 80);
            Timer.CreateAndStart(5.0, LuoKelabotti);
            
            LuoElamalaskuri();
            LuoPistelaskuri();
            
            Camera.Follow(pelaaja);
            
            Keyboard.Listen(Key.A, ButtonState.Pressed, AaniVasemmalle, null, new Vector());
            Keyboard.Listen(Key.D, ButtonState.Pressed, AaniOikealle, null, new Vector());
            Keyboard.Listen(Key.A, ButtonState.Released, LopetaAaniVasemmalle, null, new Vector());
            Keyboard.Listen(Key.D, ButtonState.Released, LopetaAaniOikealle, null, new Vector());
            
            Keyboard.Listen(Key.A, ButtonState.Down, LiikuVasemmalle, null, new Vector());
            Keyboard.Listen(Key.D, ButtonState.Down, LiikuOikealle, null, new Vector());
            Keyboard.Listen(Key.W, ButtonState.Pressed, Hyppaa, null, new Vector());
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
            
            Keyboard.Listen(Key.Space, ButtonState.Pressed, LuoPesismaila, "", new Vector(90, 10), pelaaja);
        }
        }
        
        /// <summary>
        /// aliohjelma luo tiili-palikoita, joista kenttä rakentuu
        /// </summary>
        /// <param name="paikka">palikan paikka</param>
        /// <param name="leveys">palikan korkeus</param>
        /// <param name="korkeus">palikan leveys</param>
        void LuoTaso(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
            taso.Position = paikka;
            taso.CollisionIgnoreGroup = 2;
            taso.Color = Color.Gray;
            taso.Image = TiiliseinaKuva;
            taso.Tag = "taso";
            Add(taso);
        }

        /// <summary>
        /// Aliohjelma, jossa luodaan seuraavaan kenttään vievä ovi
        /// </summary>
        /// <param name="paikka">oven paikka</param>
        /// <param name="x">oven leveys</param>
        /// <param name="y">oven korkeus</param>
        void LuoExit(Vector paikka, double x, double y)
        {
            PhysicsObject kolmio = PhysicsObject.CreateStaticObject(20, 100);
            kolmio.Position = paikka;
            kolmio.CollisionIgnoreGroup = 2;
            kolmio.Color = Color.HotPink;
            kolmio.Tag = "kolmio";
            Add(kolmio);
        }
        
        /// <summary>
        /// Aliohjelma luo pelaajaa parantavan viinapllon
        /// </summary>
        /// <param name="paikka">pullon paikka</param>
        /// <param name="x">pullon leveys</param>
        /// <param name="y">pullon korkeus</param>
        void LuoViinapullo(Vector paikka, double x, double y)
        {
            Viinapullo = PhysicsObject.CreateStaticObject(50, 50);
            Viinapullo.Position = paikka;
            Viinapullo.Color = Color.White;
            Viinapullo.Tag = "Viinapullo";
            Viinapullo.Image = ViinapulloKuva;
            Add(Viinapullo);
        }

        /// <summary>
        /// Aliohjelma luo "haamupalikan" jonka sijaintiin kelabotti myöhemmin luodaan
        /// </summary>
        /// <param name="paikka">palikan paikka</param>
        /// <param name="leveys">palikan leveys</param>
        /// <param name="korkeus">palikan korkeus</param>
        void LuoKelabottiSpawni(Vector paikka, double leveys, double korkeus)
        {
            PhysicsObject BottiSpawni = PhysicsObject.CreateStaticObject(leveys, korkeus);
            BottiSpawni.Position = paikka;
            BottiSpawni.IgnoresCollisionResponse = true;
            //BottiSpawni.CollisionIgnoreGroup = 3;
            BottiSpawni.IsVisible = false;
            BottiSpawni.IgnoresGravity = true;
            Add(BottiSpawni, -1);
            spawnipaikat.Add(BottiSpawni);
        }
        
      
        /// <summary>
        /// Aliohjelma, jossa luodaan pelaaja
        /// </summary>
        /// <param name="x">pelaajan x-koordinaatti</param>
        /// <param name="y">pelaajan y-koordinaatti</param>
        /// <returns>pelaaja</returns>
        void LuoPelaaja(Vector paikka, double x, double y)
        {
            pelaaja = new PlatformCharacter(50.0, 100.0);
            //pelaaja.Image = PelaajaKuva;
            pelaaja.Mass = 1.0;
            pelaaja.Shape = Shape.Rectangle;
            pelaaja.Color = Color.Black;
            pelaaja.Position = paikka;
            pelaaja.CollisionIgnoreGroup = 1;
            pelaaja.AnimWalk = new Animation(HenkiloSaapastelu);
            pelaaja.AnimIdle = new Animation(PaikallaanAnimaatio);
            pelaaja.AnimWalk.FPS = 3;
            pelaaja.AnimFall = new Animation(PutoamisAnimaatio);
            pelaaja.AnimFall.FPS = 12;
            pelaaja.LoopFallAnim = true;
          
            // Törmäyksen käsittelijä, mitä tapahtuu, kun pelaaja osuu kelabotiin 
            //Siis pelaajan elämien väheneminen VahnigoitaPelaajaa-aliohjelman mukaisesti 
            AddCollisionHandler(pelaaja, "Kelabotti", VahingoitaPelaajaa);
            AddCollisionHandler(pelaaja, "Viinapullo", ParannaPelaajaa);
            AddCollisionHandler(pelaaja, "kappale", VahingoitaPelaajaa);
            AddCollisionHandler(pelaaja, "kolmio", KentanVaihto);
            //AddCollisionHandler(kappale, "taso", TuhoaKappale);
            
            Add(pelaaja);
        }
        
        /// <summary>
        /// Aliohjelma, joka luo pelaajalle elämälaskurin ja sen nimen: "Mielenterveys"
        /// </summary>
        void LuoElamalaskuri()
        {
            pelaajanelamaLaskuri = new DoubleMeter(100.0);
            pelaajanelamaLaskuri.MaxValue = 100;
            pelaajanelamaLaskuri.LowerLimit += ElamaLoppui;

            ProgressBar elamapalkki = new ProgressBar(150, 18);
            elamapalkki.X = Screen.Left + 110;
            elamapalkki.Y = Screen.Bottom + 50;
            elamapalkki.BindTo(pelaajanelamaLaskuri);
            elamapalkki.Color = Color.Red;
            elamapalkki.BarColor = Color.Green;
            elamapalkki.BorderColor = Color.Black;
            Label tekstikentta = new Label(200, 10, "Mielenterveys");
            tekstikentta.X = Screen.Left + 100;
            tekstikentta.Y = Screen.Bottom + 70;
            tekstikentta.Font = new Font(20);
            Add(tekstikentta);
            Add(elamapalkki);
        }
        

       
        /// <summary>
        /// Aliohjelma, joka käsittelee pelaajan kuoleman ja ilmoittaa pelin loppumisen
        /// </summary>
        void ElamaLoppui()
        {
            Label Kuolema = new Label(800, 60, "Toimeentulohakemus hylätty.");
            Kuolema.Y = 140;
            Kuolema.Font = new Font(60);
            Kuolema.TextColor = Color.Red;
            //Kuolema.Color = Color.Black;
            Add(Kuolema);
        }
        

        /// <summary>
        /// Pesismailan luominen
        /// </summary>
        /// <param name="koko">Pesismailan koko</param>
        /// <param name="pelaaja">kohde, johon Pesismaila liitetään</param>
        void LuoPesismaila(Vector koko, PlatformCharacter pelaaja)
        {
            PhysicsObject Pesismaila = new PhysicsObject(koko.X, koko.Y);
            Pesismaila.Position = pelaaja.Position + new Vector(80*pelaaja.FacingDirection.GetVector().X, 10);
            Pesismaila.CollisionIgnoreGroup = 1;
            Pesismaila.Mass = 1;
            Pesismaila.Tag = "Pesismaila";
            Add(Pesismaila);
            Pesismaila.LifetimeLeft = TimeSpan.FromSeconds(0.075);
            Sound Lyontiaani2 = Lyontiaani.CreateSound();
            Lyontiaani2.Volume = 1;
            Lyontiaani2.Play();
            Pesismaila.IgnoresGravity = true;
            
            //PISTEET ei elämät
            AddCollisionHandler(Pesismaila, "Kelabotti", KunVihollinenTuhoutuu);
        }

        
        /// <summary>
        /// Tämä funktio luo Kelabotti-vihun ja sille annetaan pelaajan seuraamisaivot
        /// </summary>
        /// <returns>Kelabotti</returns>
        void LuoKelabotti()
        {
            PhysicsObject spawni = spawnipaikat[RandomGen.NextInt(spawnipaikat.Count)];
            PlatformCharacter Kelabotti = new PlatformCharacter(50.0, 100.0);
            Kelabotti.Color = Color.Orange;
            Kelabotti.Image = KelabottiKuva;
            Kelabotti.CollisionIgnoreGroup = 3;
            Kelabotinelamalaskuri = new IntMeter(5); // Alussa on 5 elämää 
            Kelabotti.Tag = "Kelabotti";
            Kelabotti.Position = spawni.Position;
            Kelabotti.AnimWalk = new Animation(KelabottiKavely);
            Kelabotti.AnimWalk.FPS = 3;
            Add(Kelabotti);
            
            FollowerBrain seuraajanAivot = new FollowerBrain(pelaaja);
            seuraajanAivot.Speed = 100;

            Kelabotti.Brain = seuraajanAivot;
            
            Timer heittoajastin = new Timer();
            heittoajastin.Interval = 2;
            heittoajastin.Timeout += delegate() { KelabotinAmmus(Kelabotti, "pelaaja"); };
            heittoajastin.Start();
            
            Kelabotti.Destroyed += heittoajastin.Stop;
        }
        
       /// <summary>
       /// Aliohjelma, jossa käsitellään Kelabotin heittämä ammus
       /// </summary>
       /// <param name="heittavaOlio">Kelabotti heittää ammuksen</param>
       /// <param name="kohdeolionTunniste">pelaajan tunniste</param>
        public void KelabotinAmmus(PhysicsObject heittavaOlio, string kohdeolionTunniste)
        {
            PhysicsObject kappale = new PhysicsObject(15, 15, Shape.Triangle);
            kappale.Hit(new Vector(180*pelaaja.FacingDirection.GetVector().X, 10));
            //kappale.Hit(new Vector());
            kappale.Position = heittavaOlio.Position;
            kappale.CollisionIgnoreGroup = 3;
            kappale.Color = Color.Silver;
            kappale.Tag = "kappale";
            kappale.MaximumLifetime = TimeSpan.FromSeconds(1.5);
            Add(kappale);
        }
/*
        public void TuhoaKappale(PhysicsObject kappale, PhysicsObject taso)
        {
            kappale.Destroy();
        }
*/
        
        /// <summary>
        /// Tämä aliohjelma käsittlee sen, kun pelaaja osuu Kelabottiin
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="Kelabotti">pelin vihu; Kelabotti</param>
        void VahingoitaPelaajaa(PhysicsObject pelaaja, PhysicsObject kappale)
        {
            int luku = RandomGen.NextInt(19, 34);
            pelaajanelamaLaskuri.Value = pelaajanelamaLaskuri - luku; // Vähentää pelaajan elämää satunnaisen määrän
            
            if (pelaajanelamaLaskuri.Value <= 0) //jos laskuri menee nollaan... 
            { 
                pelaaja.Destroy(); //...ja pelaaja tuhotaan
                //Pesismaila.Destroy();//...myös pesismaila
            } 
        }
        
        /// <summary>
        /// Aliohjelma, joka parantaa pelaajan, kun tämä koskee viinapulloa
        /// </summary>
        /// <param name="pelaaja">kohde, joka parannetaan</param>
        /// <param name="Viinapullo">parantava esine</param>
        void ParannaPelaajaa(PhysicsObject pelaaja, PhysicsObject Viinapullo)
        {
            //Viinapullo.CollisionIgnoreGroup = 1;
            
            if (pelaajanelamaLaskuri.Value < 99)
            {
                //Viinapullo.CollisionIgnoreGroup = 0;
                Viinapulloaani.Play();
                
                pelaajanelamaLaskuri.Value = pelaajanelamaLaskuri.Value + 20;
                pisteet.Value += 20;
                Viinapullo.Destroy();
            }
        }
        
        /// <summary>
        /// Luo pistelaskurin pisteille, joita saa kun hakkaa kelabotteja
        /// </summary>
        void LuoPistelaskuri() 
        { 
            Label opintoNopat = new Label(150, 50, "Opintonopat:");
            opintoNopat.X = Screen.Left + 100;
            opintoNopat.Y = Screen.Bottom + 93;
            opintoNopat.Font = new Font(20);
            opintoNopat.TextColor = Color.Black;
            Add(opintoNopat);
            
            pisteet = new IntMeter(0); // Aloitetaan nollasta
            pisteNaytto = new Label(); 
            pisteNaytto.X = Screen.Left + 170;  
            pisteNaytto.Y = Screen.Bottom + 95;  
            pisteNaytto.TextColor = Color.OrangeRed; 
            pisteNaytto.Font = new Font(20); 
            pisteNaytto.BindTo(pisteet); // Päivitetään teksti automaattisesti 
            Add(pisteNaytto);
        } 
        
        /// <summary>
        /// Käsittelee kelabotin tuhoamisen ja antaa siitö pisteitä
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="kelabotti">pelin vihu; Kelabotti</param>
        void KunVihollinenTuhoutuu(PhysicsObject pelaaja, PhysicsObject kelabotti)
        {
            Keladamageaani.Play();
            pisteet.Value += 100; // Lisää esim. 100 pistettä, kun botti tuhotaan
            
            kelabotti.Destroy(); // Poistaa botin näytöltä
        }

        /// <summary>
        /// Aliohjelma, jossa käsitellään kentän vaihto
        /// </summary>
        /// <param name="pelaaja">pelaaja</param>
        /// <param name="kolmio">esine, josta kenttä vaihtuu</param>
        void KentanVaihto(PhysicsObject pelaaja, PhysicsObject kolmio)
        {
            if (pisteet.Value >= 300)
            
            {
                kenttanumero++;
                LuoKentta(kenttanumero);
            }
        }
    }

        
    


